/* Define macros here to suppress functions from the API being built */

#if 1

#define SKIP_Oct6100AdpcmChanOpenDef 1
#define SKIP_Oct6100AdpcmChanOpen 1
#define SKIP_Oct6100AdpcmChanCloseDef 1
#define SKIP_Oct6100AdpcmChanClose 1
#define SKIP_Oct6100AdpcmChanOpenSer 1
#define SKIP_Oct6100ApiCheckAdpcmChanParams 1
#define SKIP_Oct6100ApiReserveAdpcmChanResources 1
#define SKIP_Oct6100ApiWriteAdpcmChanStructs 1
#define SKIP_Oct6100ApiUpdateAdpcmChanEntry 1
#define SKIP_Oct6100AdpcmChanCloseSer 1
#define SKIP_Oct6100ApiAssertAdpcmChanParams 1
#define SKIP_Oct6100ApiInvalidateAdpcmChanStructs 1
#define SKIP_Oct6100ApiReleaseAdpcmChanResources 1
#define SKIP_Oct6100ApiReserveAdpcmChanEntry 1
#define SKIP_Oct6100ApiReleaseAdpcmChanEntry 1
#define SKIP_Oct6100ChannelCloseDef 1
#define SKIP_Oct6100ChannelClose 1
#define SKIP_Oct6100ChannelCreateBiDirDef 1
#define SKIP_Oct6100ChannelCreateBiDir 1
#define SKIP_Oct6100ChannelDestroyBiDirDef 1
#define SKIP_Oct6100ChannelDestroyBiDir 1
#define SKIP_Oct6100ChannelBroadcastTsstAddDef 1
#define SKIP_Oct6100ChannelBroadcastTsstAdd 1
#define SKIP_Oct6100ChannelBroadcastTsstRemove 1
#define SKIP_Oct6100ChannelGetStatsDef 1
#define SKIP_Oct6100ChannelGetStats 1
#define SKIP_Oct6100ChannelMuteDef 1
#define SKIP_Oct6100ChannelMute 1
#define SKIP_Oct6100ChannelUnMuteDef 1
#define SKIP_Oct6100ChannelUnMute 1
#define SKIP_Oct6100ChannelCloseSer 1
#define SKIP_Oct6100ApiAssertChannelParams 1
#define SKIP_Oct6100ApiInvalidateChannelStructs 1
#define SKIP_Oct6100ApiReleaseChannelResources 1
#define SKIP_Oct6100ChannelBroadcastTsstAddSer 1
#define SKIP_Oct6100ApiCheckChanTsstAddParams 1
#define SKIP_Oct6100ApiReserveTsstAddResources 1
#define SKIP_Oct6100ApiWriteTsstAddStructs 1
#define SKIP_Oct6100ApiUpdateTsstAddChanEntry 1
#define SKIP_Oct6100ApiChannelGetStatsSer 1
#define SKIP_Oct6100ApiReserveBiDirChanEntry 1
#define SKIP_Oct6100ApiReleaseBiDirChanEntry 1
#define SKIP_Oct6100ApiRetrieveNlpConfDword 1
#define SKIP_Oct6100ApiSaveNlpConfDword 1
#define SKIP_Oct6100ChannelCreateBiDirSer 1
#define SKIP_Oct6100ApiCheckChannelCreateBiDirParams 1
#define SKIP_Oct6100ApiReserveChannelCreateBiDirResources 1
#define SKIP_Oct6100ApiWriteChannelCreateBiDirStructs 1
#define SKIP_Oct6100ApiUpdateBiDirChannelEntry 1
#define SKIP_Oct6100ChannelDestroyBiDirSer 1
#define SKIP_Oct6100ApiAssertDestroyBiDirChanParams 1
#define SKIP_Oct6100ApiInvalidateBiDirChannelStructs 1
#define SKIP_Oct6100ApiReleaseBiDirChannelResources 1
#define SKIP_Oct6100ApiOctFloatToDbEnergyByte 1
#define SKIP_Oct6100ApiOctFloatToDbEnergyHalf 1
#define SKIP_Oct6100ChannelMuteSer 1
#define SKIP_Oct6100ApiAssertChannelMuteParams 1
#define SKIP_Oct6100ChannelUnMuteSer 1
#define SKIP_Oct6100ApiAssertChannelUnMuteParams 1
#define SKIP_Oct6100ApiMuteSinWithFeatures 1
#define SKIP_Oct6100ApiMuteChannelPorts 1
#define SKIP_Oct6100CreateLocalInstanceDef 1
#define SKIP_Oct6100CreateLocalInstance 1
#define SKIP_Oct6100DestroyLocalInstanceDef 1
#define SKIP_Oct6100DestroyLocalInstance 1
#define SKIP_Oct6100GetHwRevisionDef 1
#define SKIP_Oct6100GetHwRevision 1
#define SKIP_Oct6100FreeResourcesDef 1
#define SKIP_Oct6100FreeResources 1
#define SKIP_Oct6100ProductionBistDef 1
#define SKIP_Oct6100ProductionBist 1
#define SKIP_Oct6100ApiGetVersionDef 1
#define SKIP_Oct6100ApiGetVersion 1
#define SKIP_Oct6100FreeResourcesSer 1
#define SKIP_Oct6100ProductionBistSer 1
#define SKIP_Oct6100ChipGetStatsDef 1
#define SKIP_Oct6100ChipGetStats 1
#define SKIP_Oct6100ChipGetImageInfoDef 1
#define SKIP_Oct6100ChipGetImageInfo 1
#define SKIP_Oct6100ChipGetStatsSer 1
#define SKIP_Oct6100ConfBridgeOpenDef 1
#define SKIP_Oct6100ConfBridgeOpen 1
#define SKIP_Oct6100ConfBridgeClose 1
#define SKIP_Oct6100ConfBridgeChanAddDef 1
#define SKIP_Oct6100ConfBridgeChanAdd 1
#define SKIP_Oct6100ConfBridgeChanRemove 1
#define SKIP_Oct6100ConfBridgeChanMuteDef 1
#define SKIP_Oct6100ConfBridgeChanMute 1
#define SKIP_Oct6100ConfBridgeChanUnMuteDef 1
#define SKIP_Oct6100ConfBridgeChanUnMute 1
#define SKIP_Oct6100ConfBridgeDominantSpeakerSetDef 1
#define SKIP_Oct6100ConfBridgeDominantSpeakerSet 1
#define SKIP_Oct6100ConfBridgeMaskChangeDef 1
#define SKIP_Oct6100ConfBridgeMaskChange 1
#define SKIP_Oct6100ConfBridgeGetStatsDef 1
#define SKIP_Oct6100ConfBridgeGetStats 1
#define SKIP_Oct6100ConfBridgeOpenSer 1
#define SKIP_Oct6100ApiCheckBridgeParams 1
#define SKIP_Oct6100ApiReserveBridgeResources 1
#define SKIP_Oct6100ApiUpdateBridgeEntry 1
#define SKIP_Oct6100ConfBridgeChanAddSer 1
#define SKIP_Oct6100ApiCheckBridgeAddParams 1
#define SKIP_Oct6100ApiReserveBridgeAddResources 1
#define SKIP_Oct6100ApiBridgeEventAdd 1
#define SKIP_Oct6100ApiBridgeAddParticipantToChannel 1
#define SKIP_Oct6100ConfBridgeChanMuteSer 1
#define SKIP_Oct6100ApiCheckBridgeMuteParams 1
#define SKIP_Oct6100ApiUpdateBridgeMuteResources 1
#define SKIP_Oct6100ConfBridgeChanUnMuteSer 1
#define SKIP_Oct6100ApiCheckBridgeUnMuteParams 1
#define SKIP_Oct6100ApiUpdateBridgeUnMuteResources 1
#define SKIP_Oct6100ConfBridgeDominantSpeakerSetSer 1
#define SKIP_Oct6100ApiCheckBridgeDominantSpeakerParams 1
#define SKIP_Oct6100ApiUpdateBridgeDominantSpeakerResources 1
#define SKIP_Oct6100ConfBridgeMaskChangeSer 1
#define SKIP_Oct6100ApiCheckBridgeMaskChangeParams 1
#define SKIP_Oct6100ApiUpdateMaskModifyResources 1
#define SKIP_Oct6100ApiBridgeUpdateMask 1
#define SKIP_Oct6100ConfBridgeGetStatsSer 1
#define SKIP_Oct6100ApiReserveBridgeEntry 1
#define SKIP_Oct6100ApiReserveFlexConfParticipantEntry 1
#define SKIP_Oct6100DebugSelectChannelDef 1
#define SKIP_Oct6100DebugSelectChannel 1
#define SKIP_Oct6100DebugGetDataDef 1
#define SKIP_Oct6100DebugGetData 1
#define SKIP_Oct6100DebugSelectChannelSer 1
#define SKIP_Oct6100DebugGetDataSer 1
#define SKIP_Oct6100BufferPlayoutGetEventDef 1
#define SKIP_Oct6100BufferPlayoutGetEvent 1
#define SKIP_Oct6100BufferPlayoutGetEventSer 1
#define SKIP_Oct6100InterruptConfigureDef 1
#define SKIP_Oct6100InterruptConfigure 1
#define SKIP_Oct6100ApiReserveBufferPlayoutMemoryNode 1
#define SKIP_Oct6100ApiReleaseBufferPlayoutMemoryNode 1
#define SKIP_Oct6100ApiReserveBufferPlayoutMemory 1
#define SKIP_Oct6100ApiReleaseBufferPlayoutMemory 1
#define SKIP_Oct6100ApiCreateFeatureMask 1
#define SKIP_Oct6100MixerCopyEventCreateDef 1
#define SKIP_Oct6100MixerCopyEventCreate 1
#define SKIP_Oct6100MixerCopyEventDestroyDef 1
#define SKIP_Oct6100MixerCopyEventDestroy 1
#define SKIP_Oct6100MixerCopyEventCreateSer 1
#define SKIP_Oct6100ApiCheckCopyEventCreateParams 1
#define SKIP_Oct6100ApiReserveCopyEventCreateResources 1
#define SKIP_Oct6100ApiWriteCopyEventCreateStructs 1
#define SKIP_Oct6100ApiUpdateCopyEventCreateEntry 1
#define SKIP_Oct6100MixerCopyEventDestroySer 1
#define SKIP_Oct6100ApiAssertCopyEventDestroyParams 1
#define SKIP_Oct6100ApiInvalidateCopyEventStructs 1
#define SKIP_Oct6100ApiReleaseCopyEventResources 1
#define SKIP_Oct6100ApiReserveCopyEventEntry 1
#define SKIP_Oct6100ApiReleaseCopyEventEntry 1
#define SKIP_Oct6100PhasingTsstOpenDef 1
#define SKIP_Oct6100PhasingTsstOpen 1
#define SKIP_Oct6100PhasingTsstCloseDef 1
#define SKIP_Oct6100PhasingTsstClose 1
#define SKIP_Oct6100PhasingTsstOpenSer 1
#define SKIP_Oct6100ApiCheckPhasingParams 1
#define SKIP_Oct6100ApiReservePhasingResources 1
#define SKIP_Oct6100ApiWritePhasingStructs 1
#define SKIP_Oct6100ApiUpdatePhasingEntry 1
#define SKIP_Oct6100PhasingTsstCloseSer 1
#define SKIP_Oct6100ApiAssertPhasingParams 1
#define SKIP_Oct6100ApiInvalidatePhasingStructs 1
#define SKIP_Oct6100ApiReleasePhasingResources 1
#define SKIP_Oct6100ApiReservePhasingEntry 1
#define SKIP_Oct6100ApiReleasePhasingEntry 1
#define SKIP_Oct6100BufferPlayoutLoadDef 1
#define SKIP_Oct6100BufferPlayoutLoad 1
#define SKIP_Oct6100BufferPlayoutLoadBlockInitDef 1
#define SKIP_Oct6100BufferPlayoutLoadBlockInit 1
#define SKIP_Oct6100BufferPlayoutLoadBlockDef 1
#define SKIP_Oct6100BufferPlayoutLoadBlock 1
#define SKIP_Oct6100BufferPlayoutUnloadDef 1
#define SKIP_Oct6100BufferPlayoutUnload 1
#define SKIP_Oct6100BufferPlayoutAddDef 1
#define SKIP_Oct6100BufferPlayoutAdd 1
#define SKIP_Oct6100BufferPlayoutStartDef 1
#define SKIP_Oct6100BufferPlayoutStart 1
#define SKIP_Oct6100BufferPlayoutStop 1
#define SKIP_Oct6100BufferLoadSer 1
#define SKIP_Oct6100BufferLoadBlockInitSer 1
#define SKIP_Oct6100BufferLoadBlockSer 1
#define SKIP_Oct6100ApiCheckBufferParams 1
#define SKIP_Oct6100ApiCheckBufferLoadBlockParams 1
#define SKIP_Oct6100ApiReserveBufferResources 1
#define SKIP_Oct6100ApiWriteBufferInMemory 1
#define SKIP_Oct6100ApiUpdateBufferEntry 1
#define SKIP_Oct6100BufferUnloadSer 1
#define SKIP_Oct6100ApiAssertBufferParams 1
#define SKIP_Oct6100ApiReleaseBufferResources 1
#define SKIP_Oct6100BufferPlayoutAddSer 1
#define SKIP_Oct6100ApiCheckPlayoutAddParams 1
#define SKIP_Oct6100ApiWriteBufferAddStructs 1
#define SKIP_Oct6100BufferPlayoutStartSer 1
#define SKIP_Oct6100ApiCheckPlayoutStartParams 1
#define SKIP_Oct6100ApiWriteChanPlayoutStructs 1
#define SKIP_Oct6100ApiReserveBufPlayoutListEntry 1
#define SKIP_Oct6100ApiReleaseBufPlayoutListEntry 1
#define SKIP_Oct6100RemoteDebugDef 1
#define SKIP_Oct6100RemoteDebug 1
#define SKIP_Oct6100ApiCheckEndianDetectField 1
#define SKIP_Oct6100ApiCalculateChecksum 1
#define SKIP_Oct6100ApiFormResponsePkt 1
#define SKIP_Oct6100ApiCheckPktCommands 1
#define SKIP_Oct6100ApiExecutePktCommands 1
#define SKIP_Oct6100ApiCheckSessionNum 1
#define SKIP_Oct6100ApiRpcReadWord 1
#define SKIP_Oct6100ApiRpcReadBurst 1
#define SKIP_Oct6100ApiRpcReadArray 1
#define SKIP_Oct6100ApiRpcWriteWord 1
#define SKIP_Oct6100ApiRpcWriteSmear 1
#define SKIP_Oct6100ApiRpcWriteBurst 1
#define SKIP_Oct6100ApiRpcSetHotChannel 1
#define SKIP_Oct6100ApiRpcGetDebugChanIndex 1
#define SKIP_Oct6100ApiRpcDisconnect 1
#define SKIP_Oct6100ToneDetectionDisable 1
#define SKIP_Oct6100TsiCnctOpenDef 1
#define SKIP_Oct6100TsiCnctOpen 1
#define SKIP_Oct6100TsiCnctCloseDef 1
#define SKIP_Oct6100TsiCnctClose 1
#define SKIP_Oct6100TsiCnctOpenSer 1
#define SKIP_Oct6100ApiCheckTsiParams 1
#define SKIP_Oct6100ApiReserveTsiResources 1
#define SKIP_Oct6100ApiWriteTsiStructs 1
#define SKIP_Oct6100ApiUpdateTsiEntry 1
#define SKIP_Oct6100TsiCnctCloseSer 1
#define SKIP_Oct6100ApiAssertTsiParams 1
#define SKIP_Oct6100ApiInvalidateTsiStructs 1
#define SKIP_Oct6100ApiReleaseTsiResources 1
#define SKIP_Oct6100ApiReserveTsiCnctEntry 1
#define SKIP_Oct6100ApiReleaseTsiCnctEntry 1
#define SKIP_Oct6100UserDriverWriteOs 1
#define SKIP_Oct6100UserDriverWriteSmearOs 1
#define SKIP_Oct6100UserDriverWriteBurstOs 1
#define SKIP_Oct6100UserDriverReadOs 1
#define SKIP_Oct6100UserDriverReadBurstOs 1
#define SKIP_OctApiBt0AddNode 1
#define SKIP_OctApiBt0AddNode2 1
#define SKIP_OctApiBt0AddNode3 1
#define SKIP_OctApiBt0AddNode4 1
#define SKIP_OctApiBt0KeyCompare 1
#define SKIP_OctApiBt0UpdateLinkDepth 1
#define SKIP_OctApiBt0Rebalance 1
#define SKIP_OctApiBt0ExternalHeavy 1
#define SKIP_OctApiBt0RemoveNode2 1
#define SKIP_OctApiBt0RemoveNode3 1
#define SKIP_OctApiBt0RemoveNode 1
#define SKIP_OctApiBt0QueryNode2 1
#define SKIP_OctApiBt0QueryNode 1
#define SKIP_OctApiBt0GetFirstNode 1
#define SKIP_OctApiBt0FindOrAddNode 1
#define SKIP_OctApiBt0AddNodeReportPrevNodeData 1
#define SKIP_OctApiLmCompare 1
#define SKIP_OctApiLmMultiply 1
#define SKIP_OctApiLmDivide 1
#define SKIP_OctApiLmShiftRight1 1
#define SKIP_OctApiLmShiftn 1
#define SKIP_OctApiLmGetMsb 1
#define SKIP_OctApiTllmAllocGetSize 1
#define SKIP_OctApiTllmAllocInit 1
#define SKIP_OctApiTllmAllocInfo 1
#define SKIP_OctApiTllmAllocAlloc 1
#define SKIP_OctApiTllmAllocDealloc 1
#define SKIP_OctApiTllmCheckTimeoutList 1
#define SKIP_OctApiLlmListGetSize 1
#define SKIP_OctApiLlmListGetItemPointer 1
#define SKIP_OctApiLlmListInit 1
#define SKIP_OctApiLlmListInfo 1
#define SKIP_OctApiLlmListCreate 1
#define SKIP_OctApiLlmListDelete 1
#define SKIP_OctApiLlmListLength 1
#define SKIP_OctApiLlmListItemData 1
#define SKIP_OctApiLlmListInsertItem 1
#define SKIP_OctApiLlmListCreateFull 1
#define SKIP_OctApiLlmListAppendItems 1
#define SKIP_OctApiLlmListAppendAndSetItems 1
#define SKIP_OctApiLlmListSetItems 1
#define SKIP_OctApiLlmListCopyData 1
#define SKIP_OctApiLlmListRemoveItem 1
#define SKIP_OctApiLlm2ListGetSize 1
#define SKIP_OctApiLlm2ListGetItemPointer 1
#define SKIP_OctApiLlm2ListInit 1
#define SKIP_OctApiLlm2ListCreate 1
#define SKIP_OctApiLlm2ListDelete 1
#define SKIP_OctApiLlm2ListLength 1
#define SKIP_OctApiLlm2ListItemData 1
#define SKIP_OctApiLlm2ListInsertItem 1
#define SKIP_OctApiLlm2ListRemoveItem 1
#define SKIP_OctApiLlmMemCpy 1

#endif

